@@ -301,7 +301,7 @@ static int pxy_fsalattr_to_fattr4(const struct attrlist *attrs, fattr4 *data)
 	struct xdr_attrs_args args;
 
 	for (i = 0; i < ARRAY_SIZE(fsal_mask2bit); i++) {
-		if (FSAL_TEST_MASK(attrs->mask, fsal_mask2bit[i].mask)) {
+		if (FSAL_TEST_MASK(attrs->valid_mask, fsal_mask2bit[i].mask)) {
 			if (fsal_mask2bit[i].fattr_bit > 31) {
 				bmap.map[1] |=
 				    1U << (fsal_mask2bit[i].fattr_bit - 32);
@@ -820,6 +820,7 @@ static int pxy_setclientid(clientid4 *resultclientid, uint32_t *lease_time)
 	nfs_argop4 arg[FSAL_CLIENTID_NB_OP_ALLOC];
 	nfs_resop4 res[FSAL_CLIENTID_NB_OP_ALLOC];
 	nfs_client_id4 nfsclientid;
+	uint64_t temp_verifier;
 	cb_client4 cbproxy;
 	char clientid_name[MAXNAMLEN + 1];
 	SETCLIENTID4resok *sok;
@@ -838,12 +839,11 @@ static int pxy_setclientid(clientid4 *resultclientid, uint32_t *lease_time)
 		 getpid());
 	nfsclientid.id.id_len = strlen(clientid_name);
 	nfsclientid.id.id_val = clientid_name;
-	if (sizeof(ServerBootTime.tv_sec) == NFS4_VERIFIER_SIZE)
-		memcpy(&nfsclientid.verifier, &ServerBootTime.tv_sec,
-		       sizeof(nfsclientid.verifier));
-	else
-		snprintf(nfsclientid.verifier, NFS4_VERIFIER_SIZE, "%08x",
-			 (int)ServerBootTime.tv_sec);
+
+	/* copy to intermediate uint64_t to 0-fill or truncate as needed */
+	temp_verifier = (uint64_t)ServerBootTime.tv_sec;
+	BUILD_BUG_ON(sizeof(nfsclientid.verifier) != sizeof(uint64_t));
+	memcpy(&nfsclientid.verifier, &temp_verifier, sizeof(uint64_t));
 
 	cbproxy.cb_program = 0;
 	cbproxy.cb_location.r_netid = "tcp";
@@ -1196,7 +1196,7 @@ static fsal_status_t pxy_create(struct fsal_obj_handle *dir_hdl,
 		 getpid(), atomic_inc_uint64_t(&fcnt));
 	owner_len = strnlen(owner_val, sizeof(owner_val));
 
-	attrib->mask &= ATTR_MODE | ATTR_OWNER | ATTR_GROUP;
+	attrib->valid_mask &= ATTR_MODE | ATTR_OWNER | ATTR_GROUP;
 	if (pxy_fsalattr_to_fattr4(attrib, &input_attr) == -1)
 		return fsalstat(ERR_FSAL_INVAL, -1);
 
@@ -1272,7 +1272,7 @@ static fsal_status_t pxy_mkdir(struct fsal_obj_handle *dir_hdl,
 	 * The caller gives us partial attributes which include mode and owner
 	 * and expects the full attributes back at the end of the call.
 	 */
-	attrib->mask &= ATTR_MODE | ATTR_OWNER | ATTR_GROUP;
+	attrib->valid_mask &= ATTR_MODE | ATTR_OWNER | ATTR_GROUP;
 	if (pxy_fsalattr_to_fattr4(attrib, &input_attr) == -1)
 		return fsalstat(ERR_FSAL_INVAL, -1);
 
@@ -1358,7 +1358,7 @@ static fsal_status_t pxy_mknod(struct fsal_obj_handle *dir_hdl,
 	 * The caller gives us partial attributes which include mode and owner
 	 * and expects the full attributes back at the end of the call.
 	 */
-	attrib->mask &= ATTR_MODE | ATTR_OWNER | ATTR_GROUP;
+	attrib->valid_mask &= ATTR_MODE | ATTR_OWNER | ATTR_GROUP;
 	if (pxy_fsalattr_to_fattr4(attrib, &input_attr) == -1)
 		return fsalstat(ERR_FSAL_INVAL, -1);
 
@@ -1418,7 +1418,7 @@ static fsal_status_t pxy_symlink(struct fsal_obj_handle *dir_hdl,
 						  fso_symlink_support))
 		return fsalstat(ERR_FSAL_NOTSUPP, ENOTSUP);
 
-	attrib->mask = ATTR_MODE;
+	attrib->valid_mask = ATTR_MODE;
 	if (pxy_fsalattr_to_fattr4(attrib, &input_attr) == -1)
 		return fsalstat(ERR_FSAL_INVAL, -1);
 
@@ -1679,9 +1679,9 @@ static fsal_status_t pxy_getattrs(struct fsal_obj_handle *obj_hdl,
 			    argoparray, resoparray);
 
 	if (rc != NFS4_OK) {
-		if (attrs->mask & ATTR_RDATTR_ERR) {
+		if (attrs->request_mask & ATTR_RDATTR_ERR) {
 			/* Caller asked for error to be visible. */
-			attrs->mask = ATTR_RDATTR_ERR;
+			attrs->valid_mask = ATTR_RDATTR_ERR;
 		}
 		return nfsstat4_to_fsal(rc);
 	}
@@ -1690,9 +1690,6 @@ static fsal_status_t pxy_getattrs(struct fsal_obj_handle *obj_hdl,
 	    NFS4_OK)
 		return fsalstat(ERR_FSAL_INVAL, 0);
 
-	/* Make sure ATTR_RDATTR_ERR is cleared on success. */
-	attrs->mask &= ~ATTR_RDATTR_ERR;
-
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
 
@@ -1720,7 +1717,16 @@ static fsal_status_t pxy_setattrs(struct fsal_obj_handle *obj_hdl,
 	nfs_argop4 argoparray[FSAL_SETATTR_NB_OP_ALLOC];
 	nfs_resop4 resoparray[FSAL_SETATTR_NB_OP_ALLOC];
 
-	if (FSAL_TEST_MASK(attrs->mask, ATTR_MODE))
+	/*
+	* No way to update CTIME using a NFSv4 SETATTR.
+	* Server will return NFS4ERR_INVAL (22).
+	* time_metadata is a readonly attribute in NFSv4 and NFSv4.1.
+	* (section 5.7 in RFC7530 or RFC5651)
+	* Nevermind : this update is useless, we prevent it.
+	*/
+	FSAL_UNSET_MASK(attrs->valid_mask, ATTR_CTIME);
+
+	if (FSAL_TEST_MASK(attrs->valid_mask, ATTR_MODE))
 		attrs->mode &= ~op_ctx->fsal_export->exp_ops.
 				fs_umask(op_ctx->fsal_export);
 
